import mocker
import os.path
import hooks


class TestJujuHost(object):
    """
    Testing object to intercept charmhelper calls and inject data, or make sure
    certain data is set.
    """
    def write_file(self, file_path, contents, owner=None, group=None,
                   perms=None):
        """
        Only write the file as requested. owner, group and perms untested.
        """
        with open(file_path, 'w') as target:
            target.write(contents)

    def mkdir(self, dir_path, owner, group, perms):
        """Not yet tested"""
        pass

    def service_start(self, service_name):
        """Not yet tested"""
        pass

    def service_reload(self, service_name):
        """Not yet tested"""
        pass

    def service_pwgen(self, service_name):
        """Not yet tested"""
        return ""

    def service_stop(self, service_name):
        """Not yet tested"""
        pass


class TestJuju(object):
    """
    Testing object to intercept juju calls and inject data, or make sure
    certain data is set.
    """

    _relation_data = {}
    _relation_ids = {}
    _relation_list = {}

    def __init__(self):
        self._config = {}

    def relation_set(self, *args, **kwargs):
        """
        Capture result of relation_set into _relation_data, which
        can then be checked later.
        """
        if "relation_id" in kwargs:
            del kwargs["relation_id"]
        self._relation_data = dict(self._relation_data, **kwargs)
        for arg in args:
            (key, value) = arg.split("=")
            self._relation_data[key] = value

    def relation_ids(self, relation_name="FIXME"):
        """
        Return expected relation_ids for tests.  Feel free to expand
        as more tests are added.
        """
        return [self._relation_ids[name] for name in self._relation_ids.keys()
                if name.find(relation_name) == 0]

    def related_units(self, relid="FIXME"):
        """
        Return expected relation_ids for tests.  Feel free to expand
        as more tests are added.
        """
        return [name for name, value in self._relation_ids.iteritems()
                if value == relid]

    def relation_list(self):
        """
        Hardcode expected relation_list for tests.  Feel free to expand
        as more tests are added.
        """
        return list(self._relation_list)

    def unit_get(self, *args):
        """
        for now the only thing this is called for is "public-address",
        so it's a simplistic return.
        """
        return "localhost"

    def local_unit(self):
        return hooks.os.environ["JUJU_UNIT_NAME"]

    def charm_dir(self):
        return hooks.os.environ["CHARM_DIR"]

    def juju_log(self, *args, **kwargs):
        pass

    def log(self, *args, **kwargs):
        pass

    def config_get(self, scope=None):
        if scope is None:
            return self.config
        else:
            return self.config[scope]

    def relation_get(self, scope=None, unit_name=None, relation_id=None):
        pass


class TestHooks(mocker.MockerTestCase):

    def setUp(self):
        hooks.hookenv = TestJuju()
        hooks.host = TestJujuHost()
        hooks.juju_log_dir = self.makeDir()
        hooks.hookenv.config = lambda: hooks.hookenv._config
        # hooks.hookenv.localunit = lambda: "localhost"
        hooks.os.environ["JUJU_UNIT_NAME"] = "landscape/1"
        hooks.os.environ["CHARM_DIR"] = os.path.abspath(
            os.path.join(os.path.dirname(__file__), os.pardir))
        hooks.postgresql_sysctl = self.makeFile()
        hooks._get_system_ram = lambda: 1024   # MB
        hooks._get_page_size = lambda: 1024 * 1024  # bytes
        self.maxDiff = None

    def assertFileContains(self, filename, lines):
        """Make sure strings exist in a file."""
        with open(filename, "r") as fp:
            contents = fp.read()
        for line in lines:
            self.assertIn(line, contents)

    def assertNotFileContains(self, filename, lines):
        """Make sure strings do not exist in a file."""
        with open(filename, "r") as fp:
            contents = fp.read()
        for line in lines:
            self.assertNotIn(line, contents)

    def assertFilesEqual(self, file1, file2):
        """Given two filenames, compare them."""
        with open(file1, "r") as fp1:
            contents1 = fp1.read()
        with open(file2, "r") as fp2:
            contents2 = fp2.read()
        self.assertEqual(contents1, contents2)


class TestHooksService(TestHooks):
    def foobar():
        pass
